#!/usr/bin/env perl
use strict;
use warnings;
use Path::Class qw/file/;

my $dat = shift or die "usage: %0 <csv>";
my $fh  = file($dat)->openr;

my %doc;

while (defined(my $line = $fh->getline)) {
    chomp $line;
    my ($doc_id, $url, $title, @tags) = split ',', $line;

    $doc{$doc_id} = {
        url   => $url,
        title => $title,
        tags  => parse_tags(@tags),
    };
}

my $input_vec = { book => 29, '梅田望夫' => 27, '将棋' => 16, '書評' => 13, '本' => 11 };

my %sim;
for my $doc_id (keys %doc) {
    $sim{$doc_id} = cos_sim($input_vec, $doc{$doc_id}->{tags});
}

for my $doc_id (sort {$sim{$b} <=> $sim{$a}} keys %sim) {
    printf "%2d: %.2f %s\n",
        $doc_id,
        $sim{$doc_id},
        $doc{$doc_id}->{title};
}

sub parse_tags {
    my $vec = {};
    for (@_) {
        my ($tag, $occ) = split ':';
        $vec->{$tag} += $occ;
    }
    return $vec;
}

sub cos_sim {
    my ($v1, $v2) = @_;
    return dot_product($v1, $v2)
        / (vec_abs($v1) * vec_abs($v2));
}

sub dot_product {
    my ($v1, $v2) = @_;
    my $v = 0;
    for (keys %$v1) {
        if ($v1->{$_} and $v2->{$_}) {
            $v += $v1->{$_} * $v2->{$_};
        }
    }
    return $v;
}

sub vec_abs {
    my $vec = shift;
    my $v = 0;
    for (values %$vec) {
        $v += $_ * $_;
    }
    return sqrt $v;
}
