#!/usr/bin/env perl
use strict;
use warnings;
use Path::Class qw/file/;

my $dat = shift or die "usage: %0 <csv>";
my $fh  = file($dat)->openr;

my (%doc, %index);

while (defined(my $line = $fh->getline)) {
    chomp $line;
    my ($doc_id, $url, $title, @tags) = split ',', $line;

    $doc{$doc_id} = {
        url     => $url,
        title   => $title,
        vec_len => 0,
    };

    for (@tags) {
        my ($tag, $occ) = split ':';

        if (not defined $index{$tag}) {
            $index{$tag} = [];
        }

        push @{$index{$tag}}, [$doc_id, $occ];
        $doc{$doc_id}->{vec_len} += $occ * $occ;
    }
}

my $input_vec = { book => 29, '梅田望夫' => 27, '将棋' => 16, '書評' => 13, '本' => 11 };

my %sim;
for my $t (keys %$input_vec) {
    for (@{$index{$t}}) {
        $sim{$_->[0]} += $input_vec->{$t} * $_->[1];
    }
}

my $l = vec_abs($input_vec);
for (keys %sim) {
    $sim{$_} /= $l * sqrt($doc{$_}->{vec_len});
}

for my $doc_id (sort {$sim{$b} <=> $sim{$a}} keys %sim) {
    printf "%2d: %.2f %s\n",
        $doc_id,
        $sim{$doc_id},
        $doc{$doc_id}->{title};
}

sub vec_abs {
    my $vec = shift;
    my $v = 0;
    for (values %$vec) {
        $v += $_ * $_;
    }
    return sqrt $v;
}
